/*
  OMPi OpenMP Compiler
  == Copyright since 2001 the OMPi Team
  == Department of Computer Science, University of Ioannina

  This file is part of OMPi.

  OMPi is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  OMPi is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with OMPi; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

/* x_arith.c -- some expression calculations */

#include <stdio.h>
#include <string.h>
#include <assert.h>
#include "symtab.h"
#include "x_arith.h"


int xar_expr_is_constant(astexpr tree)
{
  switch (tree->type)
  {
    case IDENT:
      return (0);
    case CONSTVAL:
      return (1);
    case STRING: 
      return (1);
      break;         
    case FUNCCALL:
      return (0);
    case ARRAYIDX:
      return (0);
    case DOTFIELD: 
      return (0);
    case PTRFIELD: 
      return (0);
    case BRACEDINIT: 
      return (1);
    case CASTEXPR: 
      return ( xar_expr_is_constant(tree->left) );
    case CONDEXPR: 
      return ( xar_expr_is_constant(tree->u.cond) && 
               xar_expr_is_constant(tree->left) &&
               xar_expr_is_constant(tree->right) );
    case UOP:
      switch (tree->opid)
      {
        case UOP_addr:
        case UOP_star:
        case UOP_sizeof:
        case UOP_typetrick:
          return (0);
        case UOP_sizeoftype:
          return (1);
        case UOP_neg:
        case UOP_bnot:
        case UOP_lnot:
        case UOP_inc:
        case UOP_dec:
        case UOP_paren:
	  return ( xar_expr_is_constant(tree->left) );
	default:
	  return (0);
      }
    case BOP: 
      /* Here we could check if any of the operands is 0/1/etc */
      return ( xar_expr_is_constant(tree->left) && 
               xar_expr_is_constant(tree->right) );
    case PREOP: 
    case POSTOP: 
      return ( xar_expr_is_constant(tree->left) );
    case ASS: 
    case DESIGNATED: 
    case IDXDES: 
    case DOTDES: 
    case COMMALIST:
    case SPACELIST:
      return (0);
    default:
      fprintf(stderr, "[xar_expr_is_constant]: b u g !!\n");
  }
  return (0);
}
